# Copyright (c) 2008 gocept gmbh & co. kg
# See also LICENSE.txt

import copy
import datetime
import lxml.etree
import lxml.objectify
import optparse
import pytz
import urllib
import urllib2
import xmlrpclib


class InternetX(object):

    base_url = 'https://gateway.autodns3.de/'

    def __init__(self, user, password, context):
        self.user = user
        self.password = password
        self.context = context

    def query(self, code, where):
        query = self.build_query(code, where)
        response = urllib2.urlopen(self.base_url, query)
        result = response.read()
        response.close()
        return result

    def build_query(self, code, where):
        request = self._get_base_xml()
        task = request['task']
        task['code'] = code
        E = lxml.objectify.E
        task.append(E.where(
            E.key(where['key']),
            E.operator(where['operator']),
            E.value(where['value'])))
        lxml.objectify.deannotate(request)
        # Remove the namespace decls. InternetX doesn't like those :(
        new_root = lxml.etree.Element(request.tag, request.attrib)
        new_root[:] = request.iterchildren()
        return lxml.etree.tostring(new_root,
                                   pretty_print=True,
                                   xml_declaration=True)

    def make_call(self, tree):
        xml = lxml.etree.tostring(tree)
        request = urllib2.urlopen(self.url, xml)
        return request.read()

    def _get_base_xml(self):
        E = lxml.objectify.E
        request = E.request(
            E.auth(
                E.user(self.user),
                E.password(self.password),
                E.context(self.context)),
            E.task())
        return request

function_codes = {
    101: 'create',  # create
    103: 'delete',  # delete
    104001: 'create',  # transfer in ack
    # XXX transfer out ack??
    # XXX late ack?
}



def get_relevant_changes(internetx, directory, accounting, start_date):
    """Get the changes from yesterday until now."""
    result = internetx.query('0713', where=dict(
        key='created', operator='gt', value=start_date))
    response = lxml.objectify.fromstring(result)
    data = response['result']['data']
    amount = data['summary']
    if not amount:
        return
    
    for history in data['history']:
        if history['type'] != 'domain':
            continue
        event_type = function_codes.get(int(history['function_code']))
        if event_type is None:
            continue
        created = datetime.datetime.strptime(
            str(history['created']), '%Y-%m-%d %H:%M:%S')
        created = created.replace(tzinfo=pytz.timezone('Europe/Berlin'))
        domainname = str(history['object'])
        if domainname.startswith('xn--'):
            name, tld = domainname.split('.', 1)
            domainname = '%s.%s' % (name[4:].decode('punycode'), tld)
        customer = directory.get_customer_for_domain(domainname)
        config = {'action': event_type}
        accounting.register(created.isoformat(), 'domain', domainname,
                            customer, config)


def main(args=None):
    yesterday = datetime.date.today() - datetime.timedelta(days=-1)
    parser = optparse.OptionParser()
    parser.add_option('--internetx-username',
                      type="string", nargs=1,
                      help='Username for internetx.')
    parser.add_option('--internetx-password',
                      help='Password for internetx.')
    parser.add_option('--internetx-context',
                      help='Context for internetx.')
    parser.add_option('-d', '--directory',
                      help='URL to directory (including auth)')
    parser.add_option('-a', '--accounting',
                      help='URL to accounting (including auth)')
    parser.add_option('--start-date',
                      help='Date events are evaluated from [yesterday]',
                      default=yesterday.strftime('%Y-%m-%d'))

    options, args = parser.parse_args(args)
    for required in ('internetx_username', 'internetx_password',
                     'internetx_context', 'directory', 'accounting',
                     'start_date'):
        if not getattr(options, required):
            parser.error('%s is required' % required)

    internetx = InternetX(options.internetx_username,
                          options.internetx_password,
                          options.internetx_context)
    directory = xmlrpclib.ServerProxy(options.directory)
    accounting = xmlrpclib.ServerProxy(options.accounting)
    get_relevant_changes(internetx, directory, accounting, options.start_date)
